{% extends "base.html" %}
{% block title %}{{ view.title }}{% endblock %}
{% block content %}{{ view.content }}
{% endblock %}
{% block commands %}
{% if view.user %}
<a class="command" href="{{ canonical_url(context, 'edit') }}">Edit</a>
{% endif %}
<a class="command" href="{{ canonical_url(context.parent, 'listing') }}">Browse</a>
{% endblock %}
{% block footer %}
<p class="last-edit">Last edited by <strong title="{{view.last_modified_by.committer_id}}">{{ view.last_modified_by.display_name }}</strong>
    on {{ view.last_modified_date }}</p>
{% endblock %}
