{% extends "base.html" %}
{% block title %}{{ view.title }}{% endblock %}
{% block content %}

<p>The directory you are looking for does not exist.<p>

<p><a href="{{ canonical_url(context.root_resource, 'listing') }}">
  Browse the wiki content
</a></p>

{% endblock %}
{% block commands %}
{% if view.user %}
<a href="{{ canonical_url(context, 'edit') }}">Edit</a>
{% endif %}
{% if context.get_dir_name %}
<a class="command" href="{{ canonical_url(context.root_resource, 'listing') }}">Browse</a>
{% endif %}
{% endblock %}
