#
# Copyright (C) 2010 Wikkid Developers.
#
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

"""Text to HTML formatters.

The initial implemention is ReST, but I'm hopeful that there will soon be
pygments, and other wiki text formatters.
"""
