/*
 *
 * Copyright (C) 2010 Wikkid Developers
 *
 * This software is licensed under the GNU Affero General Public License
 * version 3 (see the file LICENSE).
 */


/*********************************************
 *  Reset browser defaults & general styling
 *********************************************/
html, body, form, fieldset, legend, h1, h2, h3, h4, h5, h6, p, pre,
blockquote, ul, ol, dl, li, address, img {
    margin:0;
    padding:0;
}
body {
    font-family: "bitstream vera sans", "dejavu sans", verdana, sans-serif;
    color:#000;
    background:#f6f6f6;
}
a:link, a:visited, a:hover, a:focus, a:active {
    color:#1a7bbe;
    outline:none;
}
textarea {
    width: 95%;
    height: 400px;
    border: 3px solid #cccccc;
	padding: 5px;
    overflow:auto;
}
#commands {
    float: right;
}
table {
    border-collapse: collapse;
    border-spacing: 0;
}
img, fieldset {
    outline:none;
    border:none;
}
sup {
    vertical-align: text-top;
}
sub {
    vertical-align: text-bottom;
}
abbr, acronym {
    border-bottom: .1em dotted;
    cursor: help;
}
.clearfix {
    content:".";
    display:block;
    height:0;
    clear:both;
    visibility:hidden;
}

/*********************************************
 * Page layout
*********************************************/
#container {
    width:100%;
    color:#333;
    background:#fff;
}
#header {
    height:60px;
    padding:10px;
    color:#333;
    background:#f0cc36 url(bg-header.gif) top repeat-x;
    border-bottom:1px solid #a7a7a7;
    border-top:2px solid #f7a14e;
}
#navbar {
    height:21px;
    padding:5px;
    color:#333;
    background:#fff url(bg-navbar.gif) top repeat-x;
    border-bottom:1px solid #d3d3d3;
    border-top:1px solid #fff;
}
#main {
    padding:10px;
    color:#333;
    background:#fff;
    border-bottom:1px solid #dadada;
    min-height: 6em;
}
#footer {
    padding:10px;
    font-size:0.8em;
    color:#333;
    background:#f6f6f6;
    border-top:1px solid #fff;
}

#description {
    width: 76em;
}

.save-cancel {
    margin: 1em 0 2em 0;
}
.save-cancel input {
    font-size: 120%;
}

.message {
    color: red;
}

.preview {
    background-image: url("preview.png");
}

/***********************************
 *  Fonts
 **********************************/

h1, h2, h3, h4, h5, h6 {color:#eb6f00;}
h1 { font-size: 1.8em; }
h2, div.section h1 { font-size: 1.5em; }
h3, div.section div.section h1 { font-size: 1.4em; }
h4 { font-size: 1.3em; }
h5 { font-size: 1.2em; }
h6 { font-size: 1.1em; }
p {
    font-size: 1em;
    margin:0 0 10px;
    line-height: 1.4em;
}
p.last-edit {
    font-size: 0.6em;
    margin-left: 1em;
}
ul, ol, dl {
    list-style-position:inside;
    margin:15px;
}
ul ul, ul ol, ul dl, ol ol, ol ul, ol dl, dl dl, dl ol, dl ul {
    margin:0 0 0 15px;
}
li {
    margin:0 0 3px;
}

/***********************************
 *  Directory listings
 **********************************/

.dir-listing {
  width: 100%;
}

.dir-listing .even {
  background: #e0e0e0;
}

.dir-listing .odd {
  background: #efefef;
}

.dir-listing a {
  padding: 2px 0px 2px 20px;
  background-repeat: no-repeat;
  background-position: 2px center;
}

.dir-listing .up {
  background-image: url("ico_folder_up.gif");
}

.dir-listing .directory {
  background-image: url("ico_folder.gif");
}

.dir-listing .file {
  background-image: url("ico_file.gif");
  background-position: 4px center;
}


/***********************************
 *  Pygment styles
 **********************************/

.highlight .hll { background-color: #ffffcc }
.highlight  { background: #f8f8f8; }
.highlight .c { color: #408080; font-style: italic } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #008000; font-weight: bold } /* Keyword */
.highlight .o { color: #666666 } /* Operator */
.highlight .cm { color: #408080; font-style: italic } /* Comment.Multiline */
.highlight .cp { color: #BC7A00 } /* Comment.Preproc */
.highlight .c1 { color: #408080; font-style: italic } /* Comment.Single */
.highlight .cs { color: #408080; font-style: italic } /* Comment.Special */
.highlight .gd { color: #A00000 } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: #FF0000 } /* Generic.Error */
.highlight .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #00A000 } /* Generic.Inserted */
.highlight .go { color: #808080 } /* Generic.Output */
.highlight .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: #0040D0 } /* Generic.Traceback */
.highlight .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #008000 } /* Keyword.Pseudo */
.highlight .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #B00040 } /* Keyword.Type */
.highlight .m { color: #666666 } /* Literal.Number */
.highlight .s { color: #BA2121 } /* Literal.String */
.highlight .na { color: #7D9029 } /* Name.Attribute */
.highlight .nb { color: #008000 } /* Name.Builtin */
.highlight .nc { color: #0000FF; font-weight: bold } /* Name.Class */
.highlight .no { color: #880000 } /* Name.Constant */
.highlight .nd { color: #AA22FF } /* Name.Decorator */
.highlight .ni { color: #999999; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.highlight .nf { color: #0000FF } /* Name.Function */
.highlight .nl { color: #A0A000 } /* Name.Label */
.highlight .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #008000; font-weight: bold } /* Name.Tag */
.highlight .nv { color: #19177C } /* Name.Variable */
.highlight .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mf { color: #666666 } /* Literal.Number.Float */
.highlight .mh { color: #666666 } /* Literal.Number.Hex */
.highlight .mi { color: #666666 } /* Literal.Number.Integer */
.highlight .mo { color: #666666 } /* Literal.Number.Oct */
.highlight .sb { color: #BA2121 } /* Literal.String.Backtick */
.highlight .sc { color: #BA2121 } /* Literal.String.Char */
.highlight .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
.highlight .s2 { color: #BA2121 } /* Literal.String.Double */
.highlight .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.highlight .sh { color: #BA2121 } /* Literal.String.Heredoc */
.highlight .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
.highlight .sx { color: #008000 } /* Literal.String.Other */
.highlight .sr { color: #BB6688 } /* Literal.String.Regex */
.highlight .s1 { color: #BA2121 } /* Literal.String.Single */
.highlight .ss { color: #19177C } /* Literal.String.Symbol */
.highlight .bp { color: #008000 } /* Name.Builtin.Pseudo */
.highlight .vc { color: #19177C } /* Name.Variable.Class */
.highlight .vg { color: #19177C } /* Name.Variable.Global */
.highlight .vi { color: #19177C } /* Name.Variable.Instance */
.highlight .il { color: #666666 } /* Literal.Number.Integer.Long */

/***********************************
 *  Custom Links
 **********************************/
a[target='_blank'] {
    background: url(icon-blank.gif) right top no-repeat;
    padding-right: 13px;
}
a[href ^= "mailto:"] {
    background: url(icon-mailto.gif) right top no-repeat;
    padding-right: 13px;
}

/***********************************
 *  Header
 **********************************/
.wikkidlogo {
    float:left;
    display:inline;
    width:181px;
    height:60px;
    background: url(logo.png) center center no-repeat;
    text-indent: -300px;
}
.wikkidlogo a {
    display:block;
    line-height:60px;
    color:#000;
    text-decoration:none;
    font-size:1.6em;
}
#login, #logged {
    float:right;
    width:350px;
    color:#b85d04;
    text-align:right;
}
#login a, #login a:link, #login a:hover, #login a:visited {
    color:#b85d04;
    text-decoration:none;
    font-weight:bold;
}
#login a:hover {
    text-decoration:underline;
}
#login img {
    vertical-align:middle;
}
.user-thumbnail {
    float:right;
    display:inline;
    width:50px;
    height:50px;
    margin:0 0 0 10px;
    border:1px solid #f7b558;
}
#logged span {
    color:#000;
    font-size: 0.8em;
}
#logged span a {
    color:#000;
}

/***********************************
 *  Navigation elements
 **********************************/
.breadcrumbs {
    padding: 4px 0;
    font-size: 0.8em;
}
.breadcrumbs a, .breadcrumbs a:link, .breadcrumbs a:visited, .breadcrumbs a:hover {
    color:#333;
}
.breadcrumbs a:hover {
    color:#1a7bbe;
}
#navbar ul {
    float: right;
    margin: 4px 0;
    padding: 0;
}
#navbar li {
    float:left;
    display:inline;
    padding-top:3px;
}
#navbar li:last-child {
    padding:0;
    margin:0 0 0 10px;
}
input.finder {
    width:160px;
    font-size: 1em;
    color:#000;
    background:#fff;
    margin:0;
    padding:2px;
    border-top:solid 1px #737373;
    border-right:solid 1px #b6b6b6;
    border-bottom:solid 1px #dbdbdb;
    border-left:solid 1px #737373;
}
#navbar li a img {
    vertical-align: top;
}
#navbar li a, #navbar li a:link, #navbar li a:hover {
    float:left;
    display:inline;
    color:#1a7bbe;
    text-decoration:none;
    font-weight:bold;
    padding:0 10px;
    border-right:1px solid #ccc;
}
#navbar li a:hover {
    text-decoration:underline;
}

/***********************************
 *  Table of contents
 **********************************/
#table-of-contents {
    float:right;
    display:inline;
    width:25%;
    color:#333;
    background:#f9f9f9;
    border:1px solid #e4e3e3;
    margin:0 0 10px 10px;
}
#table-of-contents h2 {
    font-size: 1.2em;
    font-weight:normal;
    color:#484848;
    padding:5px 10px;
    margin:0 0 10px;
    border-bottom:1px solid #e4e3e3;
}
#table-of-contents ol {
    margin:0 10px 10px 10px;
}
#table-of-contents li {
    margin:0 0 5px;
}
