#
# Copyright (C) 2010 Wikkid Developers.
#
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

"""The skins that are provided for Wikkid Wiki.

The are the following defined skins:
  default
"""
