#
# Copyright (C) 2010 Wikkid Developers.
#
# This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

"""Model classes for wikkid.

Model objects are those objects that the views hang off.
"""
